@@ -93,10 +93,7 @@ class ATNConfig(object):
                 and self.precedenceFilterSuppressed==other.precedenceFilterSuppressed
 
     def __hash__(self):
-        return hash( str(self.state.stateNumber) + "/" +
-                 str(self.alt) + "/" +
-                 str(self.context) + "/" +
-                 str(self.semanticContext) )
+        return hash((self.state.stateNumber, self.alt, self.context, self.semanticContext))
 
     def __str__(self):
         return unicode(self)
@@ -132,9 +129,9 @@ class LexerATNConfig(ATNConfig):
         self.passedThroughNonGreedyDecision = False if config is None else self.checkNonGreedyDecision(config, state)
 
     def __hash__(self):
-        return hash(str(self.state.stateNumber) + str(self.alt) + str(self.context) \
-                + str(self.semanticContext) + str(1 if self.passedThroughNonGreedyDecision else 0) \
-                + str(self.lexerActionExecutor))
+        return hash((self.state.stateNumber, self.alt, self.context,
+                self.semanticContext, self.passedThroughNonGreedyDecision,
+                self.lexerActionExecutor))
 
     def __eq__(self, other):
         if self is other:
@@ -143,7 +140,7 @@ class LexerATNConfig(ATNConfig):
             return False
         if self.passedThroughNonGreedyDecision != other.passedThroughNonGreedyDecision:
             return False
-        if self.lexerActionExecutor is not other.lexerActionExecutor:
+        if not(self.lexerActionExecutor==other.lexerActionExecutor):
             return False
         return super(LexerATNConfig, self).__eq__(other)
 
