@@ -533,9 +533,6 @@ class LexerATNSimulator(ATNSimulator):
     # configuration containing an ATN rule stop state. Later, when
     # traversing the DFA, we will know which rule to accept.
     def addDFAState(self, configs):
-        # the lexer evaluates predicates on-the-fly; by this point configs
-        # should not contain any configurations with unevaluated predicates.
-        assert not configs.hasSemanticContext
 
         proposed = DFAState(configs=configs)
         firstConfigWithRuleStopState = None
@@ -586,3 +583,4 @@ class LexerATNSimulator(ATNSimulator):
             return "'" + chr(t) + "'"
 
 
+LexerATNSimulator.ERROR = DFAState(0x7FFFFFFF, ATNConfigSet())
